import { parse } from "https://deno.land/std@0.106.0/flags/mod.ts";
import { createCanvas, loadImage } from "https://deno.land/x/canvas/mod.ts";
import { CollageModes, CollageModeType, collageModeType } from "../src/common/collages.ts";
import DenoCollageModes, { CastCanvasRenderingContext, ProxyImage } from "./collages.ts";
import { CollageConfig } from "../src/common/types.ts";
import { choice, shuffle } from "../src/common/utils.ts";

const args = parse(Deno.args, {
    alias: {
        'o': 'output'
    }
});

const images: ProxyImage[] = (await Promise.all(args["_"].map((imageURL) => loadImage(imageURL.toString())))).map((image) => new ProxyImage(image));

if (images.length < 2) {
    console.error("kollagen needs at least 2 images to work.");
    Deno.exit(1);
}

const modes = new DenoCollageModes();

const modeKey: CollageModeType = args["mode"] || choice(collageModeType);
const mode = modes.modes[modeKey];

console.log(`Creating a "${mode.name}" collage from ${images.length} images...`);
console.debug(`Images: ${args["_"].join(", ")}`);

const canvas = createCanvas(args["width"] || 640, args["height"] || 640);
const context = canvas.getContext("2d");

const collageConfig: CollageConfig = {
    numImages: args["n"]
}

const shuffledImages = shuffle(images);
const segments = mode.getSegments(context as CastCanvasRenderingContext, collageConfig, shuffledImages);
mode.place(context as CastCanvasRenderingContext, shuffledImages, segments);

const output = args["output"] || "collage.jpg";
console.log(`Saving to "${output}"...`);
await Deno.writeFile(output, canvas.toBuffer());